from can_id_sheet import CANFrame, VehicleSegment, Frequency
from periodic_send_thread import PeriodicSendingTask
from multiprocessing import Process
from scapy.sendrecv import *

class GenericECU(Process):
    def __init__(self,
                 protocol: str,
                 interface: str,
                 tx_socket,  # type: SuperSocket
                 rx_socket,  # type: SuperSocket
                 vehicle_segment: VehicleSegment
                 ):
        self.msg_count = 0
        def sniff_function(pkt):
            self.msg_count += 1
            # print(pkt.show())
        Process.__init__(self)
        self.protocol = protocol
        self.interface = interface
        self.tx_socket = tx_socket
        self.rx_socket = rx_socket
        self.frames_to_send = [frame for frame in CANFrame if frame.source == vehicle_segment]
        self.name = vehicle_segment.name

        ecu_work_description = f'Work description for {self.name} ECU:\n'
        if protocol == "can":  # can
            tasks = [AsyncSniffer(opened_socket=self.rx_socket, prn=sniff_function)]
        else:  # ethernet
            tasks = [AsyncSniffer(iface=self.rx_socket.iface,
                                  prn=sniff_function,
                                  filter=f"ether dst host 0{vehicle_segment.value}:00:00:00:00:00")]

        # split frames of this ECU by sending frequency and schedule them on separate cyclic threads
        for freq in Frequency:
            frames_for_freq = list(filter(lambda x: x.interval == freq, self.frames_to_send))
            if not frames_for_freq:
                continue
            tasks.append(PeriodicSendingTask(self.protocol, interface=interface, socket=self.tx_socket, frames=frames_for_freq, interval_s=freq.value))
            ecu_work_description += f' - Created cyclic thread which will send {frames_for_freq} every {freq.value}s.\n'

        self.tasks = tasks

        print(ecu_work_description)

    def run(self):
        print(f'Hello from ECU with segment: {self.name} and socket: {self.tx_socket}.')

        for t in self.tasks:
            t.start()

        time.sleep(20)

        for t in self.tasks:
            t.join(timeout=5)

        print(f'Message count {self.msg_count} from ECU with segment: {self.name}')
