from can_id_sheet import CANFrame

from scapy.layers.can import *
from scapy.layers.l2 import Ether
from scapy.all import *
from scapy.sendrecv import send
import time

NANOSECONDS_IN_SECOND: int = 1_000_000_000


class PeriodicSendingTask(Thread):
    def __init__(self,
                 protocol: str,
                 interface: str,
                 socket,  # type: SuperSocket,
                 frames: list[CANFrame],
                 interval_s: float):
        super().__init__()
        self.daemon = True
        self.stopped = False    # maybe add a way to gracefully stop
        self.protocol = protocol
        self.interface = interface  # ethernet network interface
        self.socket = socket
        self.frames = frames
        self.interval_s = interval_s
        self.interval_ns = int(round(interval_s * NANOSECONDS_IN_SECOND))
        if protocol == "can":
            self.packets = [CAN(identifier=f.value, data=b'01020304') for f in frames]
        elif protocol == "eth":
            self.packets = [Ether(src=f.source.value, dst=f.execution.value)/b"\x01\x02\x03\x04" for f in self.frames]

    def run(self) -> None:
        msg_due_time_ns = time.perf_counter_ns()

        while not self.stopped:
            if self.protocol == "can":
                # CAN packet sending
                send(self.packets, verbose=0, socket=self.socket)
                # print(f'Sent {len(self.packets)} CAN packets at {time.time()}, frequency = {self.interval_s}s')
            elif self.protocol == "eth":
                # ethernet packet sending
                sendp(self.packets, verbose=0, iface=self.socket.iface)
                # For debug
                # for p in self.packets:
                #     p.show()
                # print(f'Sent {len(self.packets)} Ethernet packets at {time.time()}, frequency = {self.interval_s}s')

            msg_due_time_ns += self.interval_ns
            # Compensate for the time it takes to send the message
            delay_ns = msg_due_time_ns - time.perf_counter_ns()
            if delay_ns > 0:
                time.sleep(delay_ns / NANOSECONDS_IN_SECOND)
